// Interacting States Rule Family
//
// By Brian Prentice  April 2002

library InteractingStates;

uses
  SysUtils,
  Classes,
  Controls,
  Dialogs,
  Input in 'Input.pas' {InputDialog};

var
  InputDialog: TInputDialog;

function CARule(Generation,col,row,
                NW,N, NE,
                W, Me,E,
                SW,S, SE: Integer): Integer; stdcall;
var
  Counts: array[1..MaxState] of Integer;
  I, J: Integer;
begin
  if NW > MaxState then
    NW := MaxState;
  if N  > MaxState then
    N  := MaxState;
  if NE > MaxState then
    NE := MaxState;
  if W  > MaxState then
    W  := MaxState;
  if Me > MaxState then
    Me := MaxState;
  if E  > MaxState then
    E  := MaxState;
  if SW > MaxState then
    SW := MaxState;
  if S  > MaxState then
    S  := MaxState;
  if SE > MaxState then
    SE := MaxState;
  for I := 1 to MaxState do
    Counts[I] := 0;
  if NW > 0 then
    Inc(Counts[NW]);
  if N > 0 then
    Inc(Counts[N]);
  if NE > 0 then
    Inc(Counts[NE]);
  if W > 0 then
    Inc(Counts[W]);
  if E > 0 then
    Inc(Counts[E]);
  if SW > 0 then
    Inc(Counts[SW]);
  if S > 0 then
    Inc(Counts[S]);
  if SE > 0 then
    Inc(Counts[SE]);
  Result := 0;
  if Me = 0 then
    case BirthTestOperators of
    OR_GE:
      for I := 1 to MaxState do
        for J := 1 to MaxState do
        if (Birth[I, J] > 0) and (Counts[J] >= Birth[I, J]) then
        begin
          Result := I;
          Exit
        end;
    OR_EQ:
      for I := 1 to MaxState do
        for J := 1 to MaxState do
        if (Birth[I, J] > 0) and (Counts[J] = Birth[I, J]) then
        begin
          Result := I;
          Exit
        end;
    AND_GE:
      for I := 1 to MaxState do
      begin
        Result := 0;
        for J := 1 to MaxState do
          if Birth[I, J] > 0 then
          begin
            Result := I;
            Break
          end;
        for J := 1 to MaxState do
          if (Birth[I, J] > 0) and (Counts[J] < Birth[I, J]) then
          begin
            Result := 0;
            Break
          end;
        if Result = I then
          Exit
      end;
    AND_EQ:
      for I := 1 to MaxState do
      begin
        Result := 0;
        for J := 1 to MaxState do
          if Birth[I, J] > 0 then
          begin
            Result := I;
            Break
          end;
        for J := 1 to MaxState do
          if (Birth[I, J] > 0) and (Counts[J] <> Birth[I, J]) then
          begin
            Result := 0;
            Break
          end;
        if Result = I then
          Exit
      end
    end
  else
    case SurvivalTestOperators of
    OR_GE:
      for J := 1 to MaxState do
      if (Survival[Me, J] > 0) and (Counts[J] >= Survival[Me, J]) then
      begin
        Result := Me;
        Exit
      end;
    OR_EQ:
      for J := 1 to MaxState do
      if (Survival[Me, J] > 0) and (Counts[J] = Survival[Me, J]) then
      begin
        Result := Me;
        Exit
      end;
    AND_GE:
    begin
      for J := 1 to MaxState do
        if Survival[Me, J] > 0 then
        begin
          Result := Me;
          Break
        end;
      for J := 1 to MaxState do
        if (Survival[Me, J] > 0) and (Counts[J] < Survival[Me, J]) then
        begin
          Result := 0;
          Exit
        end
    end;
    AND_EQ:
    begin
      for J := 1 to MaxState do
        if Survival[Me, J] > 0 then
        begin
          Result := Me;
          Break
        end;
      for J := 1 to MaxState do
        if (Survival[Me, J] > 0) and (Counts[J] <> Survival[Me, J]) then
        begin
          Result := 0;
          Exit
        end
      end  
    end
end;

procedure CAConfig(var RuleType, CountOfColors: Integer; Rule: PChar;
                   Misc: PChar); stdcall;
begin
  SaveRuleString(Rule);
  InputDialog := TInputDialog.Create(nil);
  if InputDialog.ShowModal = mrOK then
    StrCopy(Rule, PChar(GetRuleAsString));
  InputDialog.Free;
  RuleType := 2;
  CountOfColors := MaxState + 1;
  StrCopy(Misc, '')
end;

procedure CASetup(var RuleType, CountOfColors: Integer; ColorPalette,
                  Misc: PChar); stdcall;
var
  I: Integer;
begin
  I := Pos('RULE=', Misc);
  if (I > 0) then
    InitRuleFromString(Copy(Misc, I + 5, Length(Misc) - I))
  else
    InitVars;
  RuleType := 2;
  CountOfColors := MaxState + 1;
  StrCopy(ColorPalette, '')
end;

exports
  CARule   index 1,
  CASetup  index 2,
  CAConfig index 3;

begin
end.

